#include "StdAfx.h"
#include "GradientColors.h"

#define ARR_DIR_SIZE 3 // chDIMOF(m_pClrGrad[i].direction)

GradientColors::GradientColors(UINT nColors, USHORT delta) :
   m_pClrGrad(NULL),
   m_nColors(nColors),
   m_delta(min(0xFF00, delta))
{
   Init();
}

GradientColors::GradientColors(UINT nColors) :
   m_pClrGrad(NULL),
   m_nColors(nColors),
   m_delta(0xF00)
{
   Init();
}

void GradientColors::Init() {
   _ASSERT(m_nColors > 0);

   m_pClrGrad = new CrlGradInfo[m_nColors];

   for (UINT i=0; i<m_nColors; i++) {

      m_pClrGrad[i].r = rand(0xFF00);
      m_pClrGrad[i].g = rand(0xFF00);
      m_pClrGrad[i].b = rand(0xFF00);
      m_pClrGrad[i].a = 0;//0xff00;//7F80; //    :
      //    http://msdn.microsoft.com/en-us/library/dd144957%28v=vs.85%29.aspx
      // Note that GradientFill does not use the Alpha member of the TRIVERTEX structure.
      // To use GradientFill with transparency, call GradientFill and then call AlphaBlend
      // with the desired values for the alpha channel of each vertex.

      for (UINT j=0; j<ARR_DIR_SIZE; j++) {
         const COLOR16 &byteClr = getColor_(i, (ERGB)j);
         bool &dir = m_pClrGrad[i].direction[j];
         if (0xFF00 == byteClr)
            dir = false; //      
         else
         if (0x0000 == byteClr)
            dir = true; //      
         else
            dir = !!(rand(3) & 1); // 
      }
   }
}

GradientColors::~GradientColors() {
   delete [] m_pClrGrad;
   m_pClrGrad = NULL;
}

COLOR16& GradientColors::getColor_(UINT posColor, ERGB eByte) {
   _ASSERT(posColor < m_nColors);
   switch (eByte) {
   case GradientColors::r: return m_pClrGrad[posColor].r;
   case GradientColors::g: return m_pClrGrad[posColor].g;
   case GradientColors::b: return m_pClrGrad[posColor].b;
   case GradientColors::a: return m_pClrGrad[posColor].a;
   }
   _ASSERT(FALSE);
}

void GradientColors::NextIteration() {
   //DWORD dwTC = ::GetTickCount();
   int i = rand(m_nColors-1);// (dwTC>>0) % m_nColors; //      
   int j = rand(ARR_DIR_SIZE-1);//(dwTC>>2) % ARR_DIR_SIZE; //    (R or G or B)  

   COLOR16 &byteClr = getColor_(i, (ERGB)j); // R or G or B
   bool &dir = m_pClrGrad[i].direction[j];
   int iClr = byteClr;
   if (dir) {
      iClr = min(iClr + m_delta, 0xFF00);
      if (iClr >= 0xFF00)
         dir = !dir;
   } else {
      iClr= max(iClr - m_delta, 0x0000);
      if (iClr <= 0x0000)
         dir = !dir;
   }
   byteClr = iClr;

//#ifdef _DEBUG
//   ::OutputDebugString(::Format(_T("i[%i], j[%i]: "), i, j));
//   for (UINT k=0; k<m_nColors; k++)
//      ::OutputDebugString(::Format(_T("  [%05i %05i %05i]"),
//         m_pClrGrad[k].r, m_pClrGrad[k].g, m_pClrGrad[k].b));
//   ::OutputDebugString(_T("\r\n"));
//#endif
}