////////////////////////////////////////////////////////////////////////////////
// File name: Icon.cpp
// Author: Sergey Krivulya (Ce p) - KSerg (SeregaLBN)
// e-mail: Sergey_Krivulya@UkrPost.Net
// ICQ: 72099167
// WWW page: http://kserg77.narod.ru/Icon.html
// Date: 2011 06 18
//
// Description:   
// Copyright (C) 2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Icon.h"
#include <Shellapi.h>
#pragma comment(lib, "Shell32")

//////////////////////////////////////////////////////////////////////////////
//                            Data types
//////////////////////////////////////////////////////////////////////////////
//     URL: http://download.microsoft.com/download/win95/Utility/1.0/W9XXP/EN-US/4493.exe : file ICONS.H
// Structs

#pragma pack( push )
#pragma pack( 2 )

// These first three structs represent how the icon information is stored
// when it is bound into a EXE or DLL file. Structure members are WORD
// aligned and the last member of the structure is the ID instead of
// the imageoffset.
typedef struct tagMEMICONDIRENTRY
{
   BYTE   bWidth;               // Width of the image
   BYTE   bHeight;              // Height of the image (times 2)
   BYTE   bColorCount;          // Number of colors in image (0 if >=8bpp)
   BYTE   bReserved;            // Reserved
   WORD   wPlanes;              // Color Planes
   WORD   wBitCount;            // Bits per pixel
   DWORD  dwBytesInRes;         // how many bytes in this resource?
   WORD   nID;                  // the ID
} MEMICONDIRENTRY, *LPMEMICONDIRENTRY;
typedef const MEMICONDIRENTRY * LPCMEMICONDIRENTRY;

typedef struct tagICONHEAD
{
   WORD              idReserved;   // Reserved (==0)
   WORD              idType;       // resource type (1 for icons)
   WORD              idCount;      // how many images?
} ICONHEAD, *LPICONHEAD;
typedef const ICONHEAD * LPCICONHEAD;

typedef struct tagMEMICONDIR: public ICONHEAD
{
   MEMICONDIRENTRY idEntries[1]; // the entries for each image
} MEMICONDIR, *LPMEMICONDIR;
typedef const MEMICONDIR * LPCMEMICONDIR;

// These next two structs represent how the icon information is stored
// in an ICO file.
typedef struct tagICONDIRENTRY
{
   BYTE   bWidth;               // Width of the image
   BYTE   bHeight;              // Height of the image (times 2)
   BYTE   bColorCount;          // Number of colors in image (0 if >=8bpp)
   BYTE   bReserved;            // Reserved (==0)
   WORD   wPlanes;              // ico: Color Planes  ; cur: X  " " (hotspot) 
   WORD   wBitCount;            // ico: Bits per pixel; cur: Y  " " (hotspot) 
   DWORD  dwBytesInRes;         // how many bytes in this resource?
   DWORD  dwImageOffset;        // where in the file is this image
} ICONDIRENTRY, *LPICONDIRENTRY;
typedef const ICONDIRENTRY * LPCICONDIRENTRY;

typedef struct tagICONDIR: public ICONHEAD
{
   ICONDIRENTRY idEntries[1]; // the entries for each image
} ICONDIR, *LPICONDIR;
typedef const ICONDIR * LPCICONDIR;

/** /
//    ( '' )
typedef struct tagICONIMAGE
{
   union {
      BITMAPINFO icBMInfo;
      struct {
         BITMAPINFOHEADER icHeader; // Only the following members are used: biSize, biWidth, biHeight, biPlanes, biBitCount, biSizeImage. All other members must be 0. 
         RGBQUAD icColors[1..];
      };
   };
   BYTE icXOR[1..];
   BYTE icAND[1..];
} ICONIMAGE, *LPICONIMAGE;
typedef const ICONIMAGE * LPCICONIMAGE;
/**/
#pragma pack( pop )

//////////////////////////////////////////////////////////////////////////////

namespace nsInternalIcon {
   //////////////////////////////////////////////////////////////

   /* ICO file */

   //  ICONHEAD  ICO 
   BOOL GetIcoHeader(                       IN const BYTE* &pbData, IN DWORD nbSize, OUT LPCICONHEAD &pIcoHead);
   //  ICONDIRENTRY    ICO 
   BOOL GetIcoDirEntry(IN WORD wIndexFrame, IN const BYTE* &pbData, IN DWORD nbSize, OUT LPCICONHEAD &pIcoHead, OUT LPCICONDIRENTRY &pEntry);
   //   (ICONIMAGE*)    ICO 
   BOOL GetIcoBody(    IN WORD wIndexFrame, IN const BYTE* &pbData, IN DWORD nbSize, OUT LPCICONHEAD &pIcoHead, OUT LPCICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody);


   /* HMODULE */

   //  MEMICONDIR      RT_GROUP_ICON   
   BOOL GetMemIcoDir(IN HMODULE hMod, IN LPCTSTR szResourceName, IN WORD wIDLanguage, OUT LPCMEMICONDIR &pMemIcoDir);
   //  MEMICONDIR      RT_GROUP_ICON
   BOOL GetMemIcoDir(IN HMODULE hMod, IN LPCTSTR szResourceName,                      OUT LPCMEMICONDIR &pMemIcoDir);
   //  MEMICONDIR  ()    RT_ICON
   BOOL GetMemIcoDir(IN HMODULE hMod, IN WORD wIcoId,                                 OUT LPCMEMICONDIR &pMemIcoDir);

   //  MEMICONDIR  MEMICONDIRENTRY        RT_GROUP_ICON   
   BOOL GetMemIcoDirEntry(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, IN WORD wIDLanguage, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry);
   //  MEMICONDIR  MEMICONDIRENTRY        RT_GROUP_ICON
   BOOL GetMemIcoDirEntry(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName,                      OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry);
   //  MEMICONDIR  MEMICONDIRENTRY     ()   RT_ICON
   BOOL GetMemIcoDirEntry(IN HMODULE hMod, IN WORD wIcoId,                                                      OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry);

   //  MEMICONDIR  MEMICONDIRENTRY   (ICONIMAGE*)        RT_GROUP_ICON   
   BOOL GetIcoBody(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, IN WORD wIDLanguage, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody);
   //  MEMICONDIR  MEMICONDIRENTRY   (ICONIMAGE*)        RT_GROUP_ICON
   BOOL GetIcoBody(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName,                      OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody);
   //  MEMICONDIR  MEMICONDIRENTRY   (ICONIMAGE*)     ()   RT_ICON
   BOOL GetIcoBody(IN HMODULE hMod, IN WORD wIcoId,                                                      OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody);
   //                                  (ICONIMAGE*)     ()   RT_ICON
   BOOL GetIcoBody(IN HMODULE hMod, IN WORD wIcoId,                                                                                                                     OUT const BYTE* &pbIcoBody);

   HICON LoadIcon(const BITMAPINFO &bmpInfo, DWORD nbSizeRes) {
      return ::CreateIconFromResourceEx(
         (PBYTE)&bmpInfo,
         nbSizeRes,
         TRUE, 0x00030000,
         bmpInfo.bmiHeader.biWidth,
         bmpInfo.bmiHeader.biHeight>>1,
         0);
   }
   HICON LoadIcon(LPCICONDIRENTRY pEntry, const BITMAPINFO *pBmpInfo) {
      if (!pEntry || !pBmpInfo) {
         ::SetLastError(ERROR_INVALID_PARAMETER);
         return NULL;
      }
      return ::CreateIconFromResourceEx(
         (PBYTE)pBmpInfo,
         pEntry->dwBytesInRes,
         TRUE, 0x00030000,
         pEntry->bWidth ,// pBmpInfo->bmiHeader.biWidth,
         pEntry->bHeight,// pBmpInfo->bmiHeader.biHeight>>1,
         0);
   }
   HICON LoadIcon(LPCMEMICONDIRENTRY &pEntry, const BITMAPINFO &bmpInfo) {
      return ::CreateIconFromResourceEx(
         (PBYTE)&bmpInfo,
         pEntry->dwBytesInRes,
         TRUE, 0x00030000,
         pEntry->bWidth ,// bmpInfo.bmiHeader.biWidth,
         pEntry->bHeight,// bmpInfo.bmiHeader.biHeight>>1,
         0);
   }

   namespace {
      __forceinline WORD DIBNumColors(const BITMAPINFOHEADER &bih) { // Calculates the number of entries in the color table.
         if (bih.biClrUsed) {
            return (WORD)bih.biClrUsed;
         } else {
            switch (bih.biBitCount) {
            case 1:
            case 4:
            case 8:
               return 1 << bih.biBitCount;
            default: return 0;
            }
         }
      }
      __forceinline WORD PaletteSize(const BITMAPINFOHEADER &bih) { // Calculates the number of bytes in the color table.
         return DIBNumColors(bih) * sizeof(RGBQUAD);
      }
      __forceinline BYTE* FindDIBBits(const BITMAPINFOHEADER &bih) { // Locate the image bits in a CF_DIB format DIB.
         return ((BYTE*)&bih) + bih.biSize + PaletteSize(bih);
      }
      #define WIDTHBYTES(bits)      ((((bits) + 31)>>5)<<2) // How wide, in bytes, would this many bits be, DWORD aligned?
      __forceinline DWORD BytesPerLine(const BITMAPINFOHEADER &bih) { // Calculates the number of bytes in one scan line.
         return WIDTHBYTES(bih.biWidth * bih.biPlanes * bih.biBitCount);
      }
   }

   HICON LoadIcon2(const ICONDIRENTRY &entry, const BITMAPINFO &bmpInfo) {
      //DWORD dwNumberColor = 0; //if (bmpInfo.bmiHeader.biBitCount > 16) dwNumberColor = 0; else
      //if (bmpInfo.bmiHeader.biBitCount < 16)
      //   dwNumberColor = 1 << bmpInfo.bmiHeader.biBitCount;
      //DWORD dwRgbTable = sizeof(RGBQUAD) * dwNumberColor;
      //DWORD dwSizeColor =(bmpInfo.bmiHeader.biWidth*bmpInfo.bmiHeader.biBitCount+31)/32*4 * bmpInfo.bmiHeader.biHeight;
      //DWORD dwSizeMask = (bmpInfo.bmiHeader.biWidth                             +31)/32*4 * bmpInfo.bmiHeader.biHeight;

      const BYTE *pbANDbits = FindDIBBits(bmpInfo.bmiHeader)+(entry.bHeight*BytesPerLine(bmpInfo.bmiHeader)); // pbIcoBody+sizeof(BITMAPINFOHEADER)+dwRgbTable;
      const BYTE *pbXORbits = FindDIBBits(bmpInfo.bmiHeader); // pbIcoBody+sizeof(BITMAPINFOHEADER)+dwRgbTable+dwSizeColor); //const BYTE *lpbXORbits);
      HICON hIco = ::CreateIcon(NULL,
         entry.bWidth, entry.bHeight,
         (BYTE)entry.wPlanes,//(BYTE)bmpInfo.bmiHeader.biPlanes,
         (BYTE)entry.wBitCount,//(BYTE)bmpInfo.bmiHeader.biBitCount,
         pbANDbits,
         pbXORbits);
      ::SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
      return hIco;
   }

   __forceinline BOOL CheckIcoBodySize(DWORD nbSizeIcoBody, const BITMAPINFO &bi) {
      //BITMAPINFO &bi = *(BITMAPINFO*)pbIcoBody;
      // 1.    ICONIMAGE  ICONIMAGE::icXOR
      DWORD dwOffXOR = bi.bmiHeader.biSize + PaletteSize(bi.bmiHeader);
      // 2.    ICONIMAGE  ICONIMAGE::icAND
      DWORD dwOffAND = dwOffXOR + ((bi.bmiHeader.biHeight>>1)*BytesPerLine(bi.bmiHeader));
      // 3.    ICONIMAGE   ICONIMAGE
      DWORD dwOffEND = dwOffAND + (bi.bmiHeader.biHeight>>1)*WIDTHBYTES(bi.bmiHeader.biWidth * bi.bmiHeader.biPlanes * 1/*bi.bmiHeader.biBitCount*/);
      BOOL bRes = (dwOffEND == nbSizeIcoBody);
      _ASSERTE(dwOffEND == nbSizeIcoBody);
      return bRes;
   }

   __forceinline SIZE SizeIcon(const BITMAPINFO &bi) {
      SIZE size = {bi.bmiHeader.biWidth, bi.bmiHeader.biHeight>>1};
      return size;
   }

   BOOL SaveIcoFrame(IN LPCTSTR szIcoFileFrame, IN LPCICONHEAD pIcoHead, IN LPCICONDIRENTRY pEntry, IN const BYTE* pbIcoBody) {
      DWORD dwErrCode = NO_ERROR;
      BOOL bRes = (szIcoFileFrame && pIcoHead && pEntry && pbIcoBody);
      if (!bRes) {
         dwErrCode = ERROR_INVALID_PARAMETER;
      } else {
         HANDLE hFile = ::CreateFile(
            szIcoFileFrame,
            GENERIC_WRITE,
            FILE_SHARE_READ,
            NULL,
            CREATE_ALWAYS,
            FILE_ATTRIBUTE_NORMAL,
            NULL
         );
         bRes = (hFile != INVALID_HANDLE_VALUE);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         } else {
            //     ico 
            DWORD dwNOBW = 0;
            ICONHEAD head = *pIcoHead; head.idCount = 1; //    
            ICONDIRENTRY entry2 = *pEntry; entry2.dwImageOffset = sizeof(ICONDIR); //    ICO    
            bRes = (::WriteFile(hFile, &head    , sizeof(head)       , &dwNOBW, NULL) && (sizeof(head)        == dwNOBW)) && // 
                   (::WriteFile(hFile, &entry2  , sizeof(entry2)     , &dwNOBW, NULL) && (sizeof(entry2)      == dwNOBW)) && // ICONDIRENTRY
                   (::WriteFile(hFile, pbIcoBody, entry2.dwBytesInRes, &dwNOBW, NULL) && (entry2.dwBytesInRes == dwNOBW));   //  
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
            ::CloseHandle(hFile);
         }
      }
      ::SetLastError(dwErrCode);
      return bRes;
   }

   BOOL SaveIcoFrame(IN LPCTSTR szIcoFileFrame, IN LPCMEMICONDIR &pIcoDir, IN LPCMEMICONDIRENTRY &pEntry, IN const BYTE* pbIcoBody) {
      DWORD dwErrCode = NO_ERROR;
      HANDLE hFile = ::CreateFile(
         szIcoFileFrame,
         GENERIC_WRITE,
         FILE_SHARE_READ,
         NULL,
         CREATE_ALWAYS,
         FILE_ATTRIBUTE_NORMAL,
         NULL
      );
      BOOL bRes = (hFile != INVALID_HANDLE_VALUE);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         //     ico 
         DWORD dwNOBW = 0;
         ICONHEAD head = *pIcoDir; head.idCount = 1; //    
         ICONDIRENTRY entry = {
            pEntry->bWidth,
            pEntry->bHeight,
            pEntry->bColorCount,
            pEntry->bReserved,
            pEntry->wPlanes,
            pEntry->wBitCount,
            pEntry->dwBytesInRes,
            sizeof(ICONDIR) //    ICO    
         };
         bRes = (::WriteFile(hFile, &head    , sizeof(head)      , &dwNOBW, NULL) && (sizeof(head)       == dwNOBW)) && // 
                (::WriteFile(hFile, &entry   , sizeof(entry)     , &dwNOBW, NULL) && (sizeof(entry)      == dwNOBW)) && // ICONDIRENTRY
                (::WriteFile(hFile, pbIcoBody, entry.dwBytesInRes, &dwNOBW, NULL) && (entry.dwBytesInRes == dwNOBW));   //  
         if (!bRes) {
            dwErrCode = ::GetLastError();
         }
         ::CloseHandle(hFile);
      }
      ::SetLastError(dwErrCode);
      return bRes;
   }

   BOOL SaveIco(IN LPCTSTR szIcoFileFrame, IN LPCMEMICONDIR &pIcoDir, IN const BYTE* pbIcoBody) {
      DWORD dwErrCode = NO_ERROR;
      HANDLE hFile = ::CreateFile(
         szIcoFileFrame,
         GENERIC_WRITE,
         FILE_SHARE_READ,
         NULL,
         CREATE_ALWAYS,
         FILE_ATTRIBUTE_NORMAL,
         NULL
      );
      BOOL bRes = (hFile != INVALID_HANDLE_VALUE);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         //   ico 
         DWORD dwNOBW = 0;
         WORD i=0;
         // 1.  ICONHEAD
         bRes = ::WriteFile(hFile, pIcoDir, sizeof(ICONHEAD), &dwNOBW, NULL) && (sizeof(ICONHEAD) == dwNOBW); // 
         for (i=0; bRes && (i<pIcoDir->idCount); i++) {
            // 2.  MEMICONDIRENTRY  ICONDIRENTRY
            const MEMICONDIRENTRY &entry = pIcoDir->idEntries[i];
            ICONDIRENTRY icoEntry = {
               entry.bWidth,
               entry.bHeight,
               entry.bColorCount,
               entry.bReserved,
               entry.wPlanes,
               entry.wBitCount,
               entry.dwBytesInRes,
               sizeof(ICONDIR)+i*sizeof(ICONDIRENTRY) //    ICO    
            };
            bRes = ::WriteFile(hFile, &icoEntry, sizeof(icoEntry), &dwNOBW, NULL) && (sizeof(icoEntry) == dwNOBW); // ICONDIRENTRY
         }
         DWORD dwOffset = 0;
         for (i=0; bRes && (i<pIcoDir->idCount); i++) {
            // 3.  
            const MEMICONDIRENTRY &entry = pIcoDir->idEntries[i];
            bRes = ::WriteFile(hFile, pbIcoBody+dwOffset, entry.dwBytesInRes, &dwNOBW, NULL) && (entry.dwBytesInRes == dwNOBW); //  
            dwOffset += entry.dwBytesInRes;
         }
         if (!bRes) {
            dwErrCode = ::GetLastError();
         }
         ::CloseHandle(hFile);
      }
      ::SetLastError(dwErrCode);
      return bRes;
   }

   BOOL SaveIco(IN LPCMEMICONDIR &pIcoDir, IN HMODULE hMod, IN LPCTSTR szIcoFile) {
      DWORD dwErrCode = NO_ERROR;
      BOOL bRes = (pIcoDir != NULL);
      if (!bRes) {
         dwErrCode = ERROR_INVALID_PARAMETER;
      } else {
         HANDLE hFile = ::CreateFile(
            szIcoFile,
            GENERIC_WRITE,
            FILE_SHARE_READ,
            NULL,
            CREATE_ALWAYS,
            FILE_ATTRIBUTE_NORMAL,
            NULL
         );
         bRes = (hFile != INVALID_HANDLE_VALUE);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         } else {
            BOOL bErrFile = TRUE;
            //    ico 
            DWORD dwNOBW = 0;
            // 1. 
            bRes = ::WriteFile(hFile, pIcoDir, sizeof(ICONHEAD), &dwNOBW, NULL) && (sizeof(ICONHEAD) == dwNOBW);
            if (!bRes) {
               dwErrCode = ::GetLastError();
            } else {
               WORD i;
               DWORD dwImageOffset = sizeof(ICONHEAD)+pIcoDir->idCount*sizeof(ICONDIRENTRY);
               // 2. MEMICONDIRENTRY   ICONDIRENTRY
               for (i=0; bRes && (i < pIcoDir->idCount); i++) {
                  ICONDIRENTRY entry = {
                     pIcoDir->idEntries[i].bWidth,
                     pIcoDir->idEntries[i].bHeight,
                     pIcoDir->idEntries[i].bColorCount,
                     pIcoDir->idEntries[i].bReserved,
                     pIcoDir->idEntries[i].wPlanes,
                     pIcoDir->idEntries[i].wBitCount,
                     pIcoDir->idEntries[i].dwBytesInRes,
                     dwImageOffset
                  };
                  bRes = ::WriteFile(hFile, &entry, sizeof(ICONDIRENTRY), &dwNOBW, NULL) && (sizeof(ICONDIRENTRY) == dwNOBW);
                  if (!bRes) {
                     dwErrCode = ::GetLastError();
                  } else {
                     dwImageOffset += pIcoDir->idEntries[i].dwBytesInRes;
                  }
               }
               // 3.  
               for (i=0; bRes && (i<pIcoDir->idCount); i++) {
                  DWORD nbSize = 0;
                  const BYTE *pIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(pIcoDir->idEntries[i].nID), nbSize);
                  bRes = (pIcoBody != NULL);
                  if (!bRes) {
                     dwErrCode = ::GetLastError();
                  } else {
                     bRes = (nbSize == pIcoDir->idEntries[i].dwBytesInRes);
                     if (!bRes) {
                        dwErrCode = ERROR_INVALID_DATA;
                     } else {
                        bRes = ::WriteFile(hFile, pIcoBody, nbSize, &dwNOBW, NULL) && (nbSize == dwNOBW); //  + all MEMICONDIRENTRY
                        if (!bRes) {
                           dwErrCode = ::GetLastError();
                        }
                     }
                  }
               }
               if (bRes)
                  bErrFile = FALSE;
            }
            ::CloseHandle(hFile); hFile = NULL;
            if (bErrFile)
               ::DeleteFile(szIcoFile);
         }
      }
      ::SetLastError(dwErrCode);
      return bRes;
   }

   BOOL FileAsMemoryOpen(IN LPCTSTR szFileName, OUT HANDLE &hFile, OUT HANDLE &hFileMap, OUT const BYTE* &pbData, OUT DWORD &nbSize) {
      BOOL bRes = TRUE;
      DWORD dwErrCode = NO_ERROR;
      TCHAR szFile[1024] = {0};
      TCHAR *szFilePart = NULL;
      DWORD dwLen = ::SearchPath(NULL, szFileName, NULL, 1024, szFile, &szFilePart);
      if (dwLen == 0) {
         dwErrCode = ::GetLastError();
      } else {
         hFile = ::CreateFile(
            szFile,
            GENERIC_READ,
            FILE_SHARE_READ,
            NULL,
            OPEN_EXISTING,
            0,
            NULL
         );
         if (hFile == INVALID_HANDLE_VALUE) {
            dwErrCode = ::GetLastError();
         } else {
            hFileMap = ::CreateFileMapping(hFile, NULL, PAGE_READONLY, 0,0, NULL);
            if (hFileMap == NULL) {
               dwErrCode = ::GetLastError();
            } else {
               pbData = (BYTE*)::MapViewOfFile(hFileMap, FILE_MAP_READ, 0,0,0);
               if (pbData==NULL) {
                  dwErrCode = ::GetLastError();
               } else {
                  nbSize = ::GetFileSize(hFile, NULL);
                  if (nbSize == 0)
                     dwErrCode = ::GetLastError();
                  else
                     bRes = TRUE;
               }
            }
         }
      }
      ::SetLastError(dwErrCode);
      return bRes;
   }
   BOOL FileAsMemoryClose(IN HANDLE &hFile, IN HANDLE &hFileMap, IN const BYTE* &pbData) {
      BOOL bRes = TRUE;
      if (pbData) {
         bRes = ::UnmapViewOfFile(pbData); pbData = NULL;
      }
      if (hFileMap != NULL) {
         bRes &= ::CloseHandle(hFileMap); hFileMap = NULL;
      }
      if ((hFile != INVALID_HANDLE_VALUE) && (hFile!=NULL)) {
         bRes &= ::CloseHandle(hFile); hFile = INVALID_HANDLE_VALUE;
      }
      return bRes;
   }
   //////////////////////////////////////////////////////////////
   //                  ...
   //////////////////////////////////////////////////////////////
   class CResourceNameEx_IcoGroupName: public CRcNameEx {
   public:
      LPCMEMICONDIR m_pMemIcoDir;
      LPCMEMICONDIRENTRY m_pEntry;
      WORD m_wIcoName;
      CResourceNameEx_IcoGroupName(WORD wIcoName) : m_wIcoName(wIcoName), m_pMemIcoDir(NULL), m_pEntry(NULL) {}
   };
   BOOL CALLBACK EnumIcoResGroupLangFunc(HMODULE hMod, LPCTSTR lpszType, LPCTSTR lpszName, WORD wIDLanguage, LONG lParam) {
      if (lpszType == RT_GROUP_ICON) {
         DWORD nbSize = 0;
         LPCMEMICONDIR pIcoDir = (LPCMEMICONDIR)::GetResource(hMod, RT_GROUP_ICON, lpszName, wIDLanguage, nbSize);
         DWORD dwErrCode = ::GetLastError();
         if (pIcoDir && (dwErrCode == ERROR_SUCCESS)) {
            BOOL bRes = (nbSize >= sizeof(MEMICONDIR)); //   +entry   
            CResourceNameEx_IcoGroupName &find = *(CResourceNameEx_IcoGroupName*)lParam;
            if (bRes) {
               WORD wIcons = pIcoDir->idCount; // -   
               bRes = (nbSize == (sizeof(ICONHEAD)+wIcons*sizeof(MEMICONDIRENTRY))); //  
#if 1
               //  1
               // ..     RT_ICON  -,     .
               bRes = (find.m_wIcoName >= pIcoDir->idEntries[0].nID) &&
                      (find.m_wIcoName < (pIcoDir->idEntries[0].nID + wIcons));
               if (bRes) {
                  const MEMICONDIRENTRY &entry = pIcoDir->idEntries[find.m_wIcoName-pIcoDir->idEntries[0].nID];
                  find.m_pMemIcoDir = pIcoDir;
                  find.m_pEntry     = &entry;
                  find.setName(lpszName);
                  find.setLang(wIDLanguage);
                  return FALSE; // continue
               }
#else
               //  2
               for (WORD i=0; bRes && (i<wIcons); i++) {
                  const MEMICONDIRENTRY &entry = pIcoDir->idEntries[i]; // MEMICONDIRENTRY  
                  WORD wResName = entry.nID; // resource name
                  if (find.m_wIcoName == wResName) {
                     find.m_pMemIcoDir = pIcoDir;
                     find.m_pEntry     = &entry;
                     find.setName(lpszName);
                     find.setLang(wIDLanguage);
                     return FALSE; // continue
                  }
               }
#endif
            }
         }
      }
      return TRUE;
   }

   BOOL CALLBACK EnumIcoResGroupNameFunc(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG lParam) {
      return ::EnumResourceLanguages(hMod, lpszType, lpszName, EnumIcoResGroupLangFunc, lParam);
   }

   CRcNameEx FindResourceIcoGroup(IN HMODULE hMod, IN WORD wResourceIcoName, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry) {
      CResourceNameEx_IcoGroupName find(wResourceIcoName);
      BOOL bRes = ::EnumResourceNames(hMod, RT_GROUP_ICON, EnumIcoResGroupNameFunc, (LONG_PTR)&find);
      pMemIcoDir = find.m_pMemIcoDir;
      pEntry     = find.m_pEntry;
      return find;
   }
   //////////////////////////////////////////////////////////////

} // nsInternalIcon

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL nsInternalIcon::GetIcoHeader(IN const BYTE* &pbData, IN DWORD nbSize, OUT LPCICONHEAD &pIcoHead) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = (pbData && nbSize);
   if (!bRes) {
      dwErrCode = ERROR_INVALID_PARAMETER;
   } else {
      bRes = (nbSize > (sizeof(ICONHEAD)+sizeof(ICONDIRENTRY)));
      if (!bRes) {
         dwErrCode = ERROR_INSUFFICIENT_BUFFER;
      } else {
         pIcoHead = (LPCICONHEAD)pbData;
         bRes = ((pIcoHead->idReserved == 0) && (pIcoHead->idType == 1)); //   ICO 
         if (!bRes) {
            dwErrCode = ERROR_INVALID_DATA;
            pIcoHead = NULL;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL nsInternalIcon::GetIcoDirEntry(IN WORD wIndexFrame, IN const BYTE* &pbData, IN DWORD nbSize, OUT LPCICONHEAD &pIcoHead, OUT LPCICONDIRENTRY &pEntry)
{
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetIcoHeader(pbData, nbSize, pIcoHead);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      bRes = (wIndexFrame < pIcoHead->idCount); //    
      if (!bRes)
         dwErrCode = ERROR_INVALID_PARAMETER;
      else {
         DWORD dwOffset = sizeof(ICONHEAD)+wIndexFrame*sizeof(ICONDIRENTRY); //       
         bRes = (nbSize > (dwOffset+sizeof(ICONDIRENTRY)));
         if (!bRes) {
            dwErrCode = ERROR_INSUFFICIENT_BUFFER;
         } else {
            pEntry = (LPCICONDIRENTRY)(pbData+dwOffset);
            bRes = (pEntry->dwBytesInRes >= 56); //   - 1x1. ...
            if (!bRes) {
               dwErrCode = ERROR_INVALID_DATA;
            } else {
               // ...
            }
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL nsInternalIcon::GetIcoBody(IN WORD wIndexFrame, IN const BYTE* &pbData, IN DWORD nbSize, OUT LPCICONHEAD &pIcoHead, OUT LPCICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody) {
                                //IN WORD wIndexFrame, IN LPCTSTR szIcoFile, OUT ICONHEAD &icoHead, OUT ICONDIRENTRY &entry, OUT BYTE* &pbIcoBody) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetIcoDirEntry(wIndexFrame, pbData, nbSize, pIcoHead, pEntry);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      DWORD dwOffset = pEntry->dwImageOffset; //   ico     
      bRes = (nbSize >= (dwOffset+pEntry->dwBytesInRes)) &&
             (nbSize >  (dwOffset+sizeof(BITMAPINFO)));
      if (!bRes) {
         dwErrCode = ERROR_INSUFFICIENT_BUFFER;
      } else {
         pbIcoBody = pbData+pEntry->dwImageOffset; //  
         bRes = CheckIcoBodySize(pEntry->dwBytesInRes, *(BITMAPINFO*)pbIcoBody);
         if (!bRes) {
            pbIcoBody = NULL;
            dwErrCode = ERROR_INVALID_DATA;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL nsInternalIcon::GetMemIcoDir(IN HMODULE hMod, IN LPCTSTR szResourceName, IN WORD wIDLanguage, OUT LPCMEMICONDIR &pMemIcoDir) {
   DWORD dwErrCode = NO_ERROR;
   DWORD nbSize = 0;
   pMemIcoDir = (LPCMEMICONDIR)::GetResource(hMod, RT_GROUP_ICON, szResourceName, wIDLanguage, nbSize);
   BOOL bRes = (pMemIcoDir != NULL);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      bRes = (nbSize >= sizeof(MEMICONDIR)); //   +entry   
      if (!bRes) {
         dwErrCode = ERROR_INVALID_DATA;
      } else {
         WORD wIcons = pMemIcoDir->idCount; // -   
         bRes = (nbSize == (sizeof(ICONHEAD)+wIcons*sizeof(MEMICONDIRENTRY))); //  
         if (!bRes) {
            dwErrCode = ERROR_INVALID_DATA;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetMemIcoDir(IN HMODULE hMod, IN LPCTSTR szResourceName, OUT LPCMEMICONDIR &pMemIcoDir) {
   DWORD dwErrCode = NO_ERROR;
   DWORD nbSize = 0;
   pMemIcoDir = (LPCMEMICONDIR)::GetResource(hMod, RT_GROUP_ICON, szResourceName, nbSize);
   BOOL bRes = (pMemIcoDir != NULL);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      bRes = (nbSize >= sizeof(MEMICONDIR)); //   +entry   
      if (!bRes) {
         dwErrCode = ERROR_INVALID_DATA;
      } else {
         WORD wIcons = pMemIcoDir->idCount; // -   
         bRes = (nbSize == (sizeof(ICONHEAD)+wIcons*sizeof(MEMICONDIRENTRY))); //  
         if (!bRes) {
            dwErrCode = ERROR_INVALID_DATA;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetMemIcoDir(IN HMODULE hMod, IN WORD wIcoId, OUT LPCMEMICONDIR &pMemIcoDir) {
   DWORD dwErrCode = NO_ERROR;

   //    (RT_GROUP_ICON)      (wIcoId).
   LPCMEMICONDIRENTRY pEntry = NULL;
   CRcNameEx finded = nsInternalIcon::FindResourceIcoGroup(hMod, wIcoId, pMemIcoDir, pEntry);
   BOOL bRes = finded.isValid();
   if (!bRes) {
      dwErrCode = ERROR_RESOURCE_NAME_NOT_FOUND;
   } else {
      // ...
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetMemIcoDirEntry(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, IN WORD wIDLanguage, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetMemIcoDir(hMod, szResourceName, wIDLanguage, pMemIcoDir);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      bRes = (wIndexFrame < pMemIcoDir->idCount);
      if (!bRes) {
         dwErrCode = ERROR_INVALID_PARAMETER;
      } else {
         pEntry = &(pMemIcoDir->idEntries[wIndexFrame]);
         // ...
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
      pEntry = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetMemIcoDirEntry(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetMemIcoDir(hMod, szResourceName, pMemIcoDir);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      bRes = (wIndexFrame < pMemIcoDir->idCount);
      if (!bRes) {
         dwErrCode = ERROR_INVALID_PARAMETER;
      } else {
         pEntry = &(pMemIcoDir->idEntries[wIndexFrame]);
         // ...
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
      pEntry = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetMemIcoDirEntry(IN HMODULE hMod, IN WORD wIcoId, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetMemIcoDir(hMod, wIcoId, pMemIcoDir);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      for (WORD i=0; i<pMemIcoDir->idCount; i++) {
         if (pMemIcoDir->idEntries[i].nID == wIcoId) {
            pEntry = &pMemIcoDir->idEntries[i];
            // ...
            break;
         }
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
      pEntry = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetIcoBody(IN HMODULE hMod, IN WORD wIcoId, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody) {
   DWORD dwErrCode = NO_ERROR;
   //    (RT_GROUP_ICON)      (wIcoId).
   CRcNameEx finded = nsInternalIcon::FindResourceIcoGroup(hMod, wIcoId, pMemIcoDir, pEntry);
   BOOL bRes = finded.isValid();
   if (!bRes) {
      dwErrCode = ERROR_RESOURCE_NAME_NOT_FOUND;
   } else {
      DWORD nbSize = 0;
      pbIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(wIcoId), nbSize);
      bRes = (pbIcoBody != NULL);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         bRes = (pEntry->dwBytesInRes == nbSize); //  
         if (bRes) {
            dwErrCode = ERROR_INVALID_DATA;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
      pEntry = NULL;
      pbIcoBody = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetIcoBody(IN HMODULE hMod, IN WORD wIcoId, OUT const BYTE* &pbIcoBody) {
   DWORD dwErrCode = NO_ERROR;
   DWORD nbSize = 0;
   pbIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(wIcoId), nbSize);
   BOOL bRes = (pbIcoBody != NULL);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      // ...
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pbIcoBody = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetIcoBody(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, IN WORD wIDLanguage, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetMemIcoDirEntry(hMod, wIndexFrame, szResourceName, wIDLanguage, pMemIcoDir, pEntry);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      DWORD nbSize = 0;
      pbIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(pEntry->nID), nbSize);
      bRes = (pbIcoBody != NULL);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         bRes = (pEntry->dwBytesInRes == nbSize); //  
         if (bRes) {
            dwErrCode = ERROR_INVALID_DATA;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
      pEntry = NULL;
      pbIcoBody = NULL;
   }
   return bRes;
}

BOOL nsInternalIcon::GetIcoBody(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, OUT LPCMEMICONDIR &pMemIcoDir, OUT LPCMEMICONDIRENTRY &pEntry, OUT const BYTE* &pbIcoBody) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = GetMemIcoDirEntry(hMod, wIndexFrame, szResourceName, pMemIcoDir, pEntry);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      DWORD nbSize = 0;
      pbIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(pEntry->nID), nbSize);
      bRes = (pbIcoBody != NULL);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         bRes = (pEntry->dwBytesInRes == nbSize); //  
         if (bRes) {
            dwErrCode = ERROR_INVALID_DATA;
         } else {
            // ...
         }
      }
   }
   ::SetLastError(dwErrCode);
   if (!bRes) {
      pMemIcoDir = NULL;
      pEntry = NULL;
      pbIcoBody = NULL;
   }
   return bRes;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

SIZE SizeIcon(HICON hIcon) {
   SIZE resSize = {0,0};
   ICONINFO iconInfo;
   if (::GetIconInfo(hIcon, &iconInfo)) {
      resSize.cx = iconInfo.xHotspot ? (iconInfo.xHotspot<<1) : 1;
      resSize.cy = iconInfo.yHotspot ? (iconInfo.yHotspot<<1) : 1;
#ifdef _DEBUG
   } else {
      DWORD dwErrCode = ::GetLastError();
      if (dwErrCode == NO_ERROR) {
         _ASSERT_EXPR(FALSE, L"SizeIcon: GetIconInfo return false.\r\n GetIconInfo   :(\r\n     ! ");
         ::SetLastError(dwErrCode);
      }
#endif // _DEBUG
   }
   return resSize;
}

SIZE SizeIcon(WORD wIndexFrame, LPCTSTR szIcoFile) {
   SIZE size = {0,0};
   DWORD dwErrCode = NO_ERROR;
   HANDLE hFile = INVALID_HANDLE_VALUE; HANDLE hFileMap = NULL; const BYTE* pbData = NULL; DWORD nbSize = 0;
   BOOL bRes = nsInternalIcon::FileAsMemoryOpen(szIcoFile, hFile, hFileMap, pbData, nbSize);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      LPCICONHEAD pIcoHead = NULL;
      LPCICONDIRENTRY pEntry = NULL;
      bRes = nsInternalIcon::GetIcoDirEntry(wIndexFrame, pbData, nbSize, pIcoHead, pEntry);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         size.cx = pEntry->bWidth;
         size.cy = pEntry->bHeight;
      }
   }
   nsInternalIcon::FileAsMemoryClose(hFile, hFileMap, pbData);
   ::SetLastError(dwErrCode);
   return size;
}

SIZE SizeIcon(HMODULE hMod, WORD wIndexFrame, LPCTSTR szResourceName) {
   SIZE size = {0,0};
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDirEntry(hMod, wIndexFrame, szResourceName, pIcoDir, pEntry);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      size.cx = pEntry->bWidth;
      size.cy = pEntry->bHeight;
   }
   ::SetLastError(dwErrCode);
   return size;
}

SIZE SizeIcon(HMODULE hMod, WORD wIndexFrame, LPCTSTR szResourceName, WORD wIDLanguage) {
   SIZE size = {0,0};
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDirEntry(hMod, wIndexFrame, szResourceName, wIDLanguage, pIcoDir, pEntry);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      size.cx = pEntry->bWidth;
      size.cy = pEntry->bHeight;
   }
   ::SetLastError(dwErrCode);
   return size;
}

SIZE SizeIcon(HMODULE hMod, WORD wIcoId) {
   SIZE size = {0,0};
   DWORD dwErrCode = NO_ERROR;
#if 1
   //  1
   DWORD bnSize = 0;
   LPCVOID pbIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(wIcoId), bnSize);
   if ((pbIcoBody == NULL) || (bnSize == 0)) {
      dwErrCode = ::GetLastError();
   } else {
      size = nsInternalIcon::SizeIcon(*(const LPBITMAPINFO)pbIcoBody);
   }
#else
   //  2
   //   DLL'   -  -  - ,        ...
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDirEntry(hMod, wIcoId, pIcoDir, pEntry);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      size.cx = pEntry->bWidth;
      size.cy = pEntry->bHeight;
   }
#endif
   ::SetLastError(dwErrCode);
   return size;
}

SIZE SizeIcon(WORD wIndexFrame, const BYTE *pbAllIcoFileData, DWORD nbSize) { //  (wIndexFrame)    
   SIZE size = {0,0};
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = (pbAllIcoFileData && nbSize);
   if (!bRes) {
      dwErrCode = ERROR_INVALID_PARAMETER;
   } else {
      LPCICONHEAD pIcoHead = NULL;
      LPCICONDIRENTRY pEntry = NULL;
      const BYTE* pbIcoBody = NULL;
      bRes = nsInternalIcon::GetIcoBody(wIndexFrame, pbAllIcoFileData, nbSize, pIcoHead, pEntry, pbIcoBody);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         size.cx = pEntry->bWidth;
         size.cy = pEntry->bHeight;
      }
   }
   ::SetLastError(dwErrCode);
   return size;
}

WORD GetIconLang(HMODULE hMod, LPCTSTR szResourceName) {
   WORD wRes = ::GetResourceLangCount(hMod, RT_GROUP_ICON, szResourceName);
   return wRes;
}

WORD GetIconCount(HMODULE hMod, BOOL bLang) {
   WORD wRes = 0;
   /** /
   if (bLang == FALSE) {
      TCHAR szModule[MAX_PATH] = {0};
      DWORD dwRes = ::GetModuleFileName(hMod, szModule, MAX_PATH);
      if (szModule[0]) {
         //    hMod = LoadLibraryEx("xxx.dll", NULL, LOAD_LIBRARY_AS_DATAFILE);
         wRes = ::GetResourceNameCount(hMod, RT_GROUP_ICON, bLang);
      } else {
         wRes = ::ExtractIconEx(szModule, -1, NULL, NULL, 0);
      }
   } else
   /**/
   {
      wRes = ::GetResourceNameCount(hMod, RT_GROUP_ICON, bLang);
   }
   return wRes;
}
WORD GetIconFrames(HMODULE hMod) {
   WORD wRes = GetResourceNameCount(hMod, RT_ICON, FALSE); // ..  RT_ICON    ,   FALSE
   return wRes;
}
WORD GetIconFrames(LPCTSTR szIcoFile) {
   WORD wRes = 0;
   DWORD dwErrCode = NO_ERROR;
   HANDLE hFile = INVALID_HANDLE_VALUE; HANDLE hFileMap = NULL; const BYTE* pbData = NULL; DWORD nbSize = 0;
   BOOL bRes = nsInternalIcon::FileAsMemoryOpen(szIcoFile, hFile, hFileMap, pbData, nbSize);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      LPCICONHEAD pIcoHead = NULL;
      bRes = nsInternalIcon::GetIcoHeader(pbData, nbSize, pIcoHead);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         wRes = pIcoHead->idCount;;
      }
   }
   nsInternalIcon::FileAsMemoryClose(hFile, hFileMap, pbData);
   ::SetLastError(dwErrCode);
   return wRes;
}

WORD GetIconFrames(HMODULE hMod, LPCTSTR szResourceName) {
   WORD wRes = 0;
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDir(hMod, szResourceName, pIcoDir);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      wRes = pIcoDir->idCount;
   }
   ::SetLastError(dwErrCode);
   return wRes;
}

WORD GetIconFrames(HMODULE hMod, LPCTSTR szResourceName, WORD wIDLanguage) {
   WORD wRes = 0;
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDir(hMod, szResourceName, wIDLanguage, pIcoDir);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      wRes = pIcoDir->idCount;
   }
   ::SetLastError(dwErrCode);
   return wRes;
}

WORD GetIconFrames(const BYTE *pbAllIcoFileData, DWORD nbSize) { //        
   WORD wRes = 0;
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = (pbAllIcoFileData && nbSize);
   if (!bRes) {
      dwErrCode = ERROR_INVALID_PARAMETER;
   } else {
      LPCICONHEAD pIcoHead = NULL;
      bRes = nsInternalIcon::GetIcoHeader(pbAllIcoFileData, nbSize, pIcoHead);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         wRes = pIcoHead->idCount;
      }
   }
   ::SetLastError(dwErrCode);
   return wRes;
}

BOOL GetIconGroupNames(IN HMODULE hMod, OUT CRcName *pNames, IN WORD &wLenBuf) {
   BOOL bRes = ::GetResourceNames(hMod, RT_GROUP_ICON, pNames, wLenBuf);
   return bRes;
}

BOOL GetIconGroupNamesEx(IN HMODULE hMod, OUT CRcNameEx *pNames, IN WORD &wLenBuf) {
   BOOL bRes = ::GetResourceNamesEx(hMod, RT_GROUP_ICON, pNames, wLenBuf);
   return bRes;
}

HICON LoadIcon(WORD wIndexFrame, LPCTSTR szIcoFile) {
   //return ::ExtractIcon(GetModuleHandle(NULL), wIndexFrame, szIcoFile);
   /** /
   {
      HICON hIco = NULL;
      UINT uiRes = ::ExtractIconEx(szIcoFile, -(wIndexFrame+0), NULL, &hIco, 1);
      return hIco;
   }
   /**/
   HICON hIco = NULL;
   DWORD dwErrCode = NO_ERROR;
   HANDLE hFile = INVALID_HANDLE_VALUE; HANDLE hFileMap = NULL; const BYTE* pbData = NULL; DWORD nbSize = 0;
   BOOL bRes = nsInternalIcon::FileAsMemoryOpen(szIcoFile, hFile, hFileMap, pbData, nbSize);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      hIco = LoadIcon(wIndexFrame, pbData, nbSize);
      if (!hIco)
         dwErrCode = ::GetLastError();
   }
   nsInternalIcon::FileAsMemoryClose(hFile, hFileMap, pbData);
   ::SetLastError(dwErrCode);
   return hIco;
}
HICON LoadIcon(HMODULE hMod, WORD wIndexFrame, LPCTSTR szResourceName) {
   /**/
   HICON hIco = NULL;
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   const BYTE* pbIcoBody = NULL;
   BOOL bRes = nsInternalIcon::GetIcoBody(hMod, wIndexFrame, szResourceName, pIcoDir, pEntry, pbIcoBody);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      BITMAPINFO &bmpInfo = *(BITMAPINFO*)pbIcoBody;
      hIco = nsInternalIcon::LoadIcon(pEntry, bmpInfo);
      if (!hIco)
         dwErrCode = ::GetLastError();
   }
   ::SetLastError(dwErrCode);
   return hIco;
}
HICON LoadIcon(HMODULE hMod, WORD wIndexFrame, LPCTSTR szResourceName, WORD wIDLanguage) {
   /**/
   HICON hIco = NULL;
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   const BYTE* pbIcoBody = NULL;
   BOOL bRes = nsInternalIcon::GetIcoBody(hMod, wIndexFrame, szResourceName, wIDLanguage, pIcoDir, pEntry, pbIcoBody);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      BITMAPINFO &bmpInfo = *(BITMAPINFO*)pbIcoBody;
      hIco = nsInternalIcon::LoadIcon(pEntry, bmpInfo);
      if (!hIco)
         dwErrCode = ::GetLastError();
   }
   ::SetLastError(dwErrCode);
   return hIco;
}
HICON LoadIcon(HMODULE hMod, WORD wIcoId) {
   /** /
   {
      CString strModule = GetModuleFileName(hMod);
      HICON hIco = ::ExtractAssociatedIcon(::GetModuleHandle(NULL), (LPTSTR)(LPCTSTR)strModule, &wIcoId);
      return hIco;
   }
   /** /
   HICON hIco = NULL;
   DWORD dwErrCode = NO_ERROR;
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   const BYTE* pbIcoBody = NULL;
   BOOL bRes = nsInternalIcon::GetIcoBody(hMod, wIcoId, pIcoDir, pEntry, pbIcoBody);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      BITMAPINFO &bmpInfo = *(BITMAPINFO*)pbIcoBody;
      hIco = nsInternalIcon::LoadIcon(pEntry, bmpInfo);
      if (!hIco)
         dwErrCode = ::GetLastError();
   }
   ::SetLastError(dwErrCode);
   return hIco;
   /**/
   HICON hIco = NULL;
   DWORD dwErrCode = NO_ERROR;
   DWORD nbSizeBody = 0;
   const BYTE *pbIcoBody = ::GetResource(hMod, RT_ICON, MAKEINTRESOURCE(wIcoId), nbSizeBody); //   
   if ((pbIcoBody == NULL) || (nbSizeBody < sizeof(BITMAPINFO))) {
      dwErrCode = ::GetLastError();
   } else {
      BITMAPINFO &bmpInfo = *(BITMAPINFO*)pbIcoBody;
      hIco = nsInternalIcon::LoadIcon(bmpInfo, nbSizeBody);
      if (!hIco)
         dwErrCode = ::GetLastError();
   }
   ::SetLastError(dwErrCode);
   return hIco;
}

HICON LoadIcon(WORD wIndexFrame, const BYTE *pbAllIcoFileData, DWORD nbSize) {
   HICON hIco = NULL;
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = (pbAllIcoFileData && nbSize);
   if (!bRes) {
      dwErrCode = ERROR_INVALID_PARAMETER;
   } else {
      LPCICONHEAD pIcoHead = NULL;
      LPCICONDIRENTRY pEntry = NULL;
      const BYTE* pbIcoBody = NULL;
      bRes = nsInternalIcon::GetIcoBody(wIndexFrame, pbAllIcoFileData, nbSize, pIcoHead, pEntry, pbIcoBody);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         const BITMAPINFO *pBmpInfo = (const BITMAPINFO *)pbIcoBody;
         hIco = nsInternalIcon::LoadIcon(pEntry, pBmpInfo);
         if (!hIco)
            dwErrCode = ::GetLastError();
      }
   }
   ::SetLastError(dwErrCode);
   return hIco;
}

BOOL ExtractIco(IN WORD wIndexFrame, IN LPCTSTR szIcoFile, IN LPCTSTR szIcoFileFrame) {
   DWORD dwErrCode = NO_ERROR;
   HANDLE hFile = INVALID_HANDLE_VALUE; HANDLE hFileMap = NULL; const BYTE* pbData = NULL; DWORD nbSize = 0;
   BOOL bRes = nsInternalIcon::FileAsMemoryOpen(szIcoFile, hFile, hFileMap, pbData, nbSize);
   if (!bRes) {
      dwErrCode = ::GetLastError();
   } else {
      LPCICONHEAD pIcoHead = NULL;
      LPCICONDIRENTRY pEntry = NULL;
      const BYTE* pbIcoBody = NULL;
      bRes = nsInternalIcon::GetIcoBody(wIndexFrame, pbData, nbSize, pIcoHead, pEntry, pbIcoBody);
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         bRes = nsInternalIcon::SaveIcoFrame(szIcoFileFrame, pIcoHead, pEntry, pbIcoBody);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         }
      }
   }
   nsInternalIcon::FileAsMemoryClose(hFile, hFileMap, pbData);
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL ExtractIco(IN HMODULE hMod, IN LPCTSTR szResourceName, IN LPCTSTR szIcoFile) {
   LPCMEMICONDIR pIcoDir = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDir(hMod, szResourceName, pIcoDir);
   if (bRes) {
      bRes = nsInternalIcon::SaveIco(pIcoDir, hMod, szIcoFile);
   }
   return bRes;
}

BOOL ExtractIco(IN HMODULE hMod, IN LPCTSTR szResourceName, IN WORD wIDLanguage, IN LPCTSTR szIcoFile) {
   LPCMEMICONDIR pIcoDir = NULL;
   BOOL bRes = nsInternalIcon::GetMemIcoDir(hMod, szResourceName, wIDLanguage, pIcoDir);
   if (bRes) {
      bRes = nsInternalIcon::SaveIco(pIcoDir, hMod, szIcoFile);
   }
   return bRes;
}

BOOL ExtractIco(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, IN LPCTSTR szIcoFileFrame) {
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   const BYTE *pbIcoBody = NULL;
   BOOL bRes = nsInternalIcon::GetIcoBody(hMod, wIndexFrame, szResourceName, pIcoDir, pEntry, pbIcoBody);
   if (bRes) {
      bRes = nsInternalIcon::SaveIcoFrame(szIcoFileFrame, pIcoDir, pEntry, pbIcoBody);
   }
   return bRes;
}

BOOL ExtractIco(IN HMODULE hMod, IN WORD wIndexFrame, IN LPCTSTR szResourceName, IN WORD wIDLanguage, IN LPCTSTR szIcoFileFrame) {
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   const BYTE *pbIcoBody = NULL;
   BOOL bRes = nsInternalIcon::GetIcoBody(hMod, wIndexFrame, szResourceName, wIDLanguage, pIcoDir, pEntry, pbIcoBody);
   if (bRes) {
      bRes = nsInternalIcon::SaveIcoFrame(szIcoFileFrame, pIcoDir, pEntry, pbIcoBody);
   }
   return bRes;
}

BOOL ExtractIco(IN HMODULE hMod, IN WORD wIcoId, IN LPCTSTR szIcoFileFrame) {
   LPCMEMICONDIR pIcoDir = NULL;
   LPCMEMICONDIRENTRY pEntry = NULL;
   const BYTE *pbIcoBody = NULL;
   BOOL bRes = nsInternalIcon::GetIcoBody(hMod, wIcoId, pIcoDir, pEntry, pbIcoBody);
   if (bRes) {
      bRes = nsInternalIcon::SaveIcoFrame(szIcoFileFrame, pIcoDir, pEntry, pbIcoBody);
   }
   return bRes;
}

