////////////////////////////////////////////////////////////////////////////////
// File name: Resources.cpp
// Author: Sergey Krivulya (Ce p) - KSerg (SeregaLBN)
// e-mail: Sergey_Krivulya@UkrPost.Net
// ICQ: 72099167
// Date: 2011 06 18
//
// Description:   
// Copyright (C) 2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Resources.h"
//#include "Bitmap.h"
//#include "Icon.h"

//     
const BYTE* GetResource(
   IN HMODULE hMod,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   OUT DWORD &dwSize
) {
   ::SetLastError(NO_ERROR);
   dwSize = 0;
   const BYTE* pbRes = NULL;
   HRSRC hRsrc = ::FindResource(hMod, szResourceName, szResourceType);
   if (hRsrc) {
      HGLOBAL hGlobal = ::LoadResource(hMod, hRsrc);
      if (hGlobal) {
         dwSize = ::SizeofResource(hMod, hRsrc);
         if (dwSize) {
            pbRes = (const BYTE*)::LockResource(hGlobal);
            if (!pbRes)
               dwSize = 0;
         }
      }
   }
   return pbRes;
}

//     
const BYTE* GetResource(
   IN HMODULE hMod,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   IN WORD wLanguage,
   OUT DWORD &dwSize
) {
   ::SetLastError(NO_ERROR);
   dwSize = 0;
   const BYTE* pbRes = NULL;
   HRSRC hRsrc = ::FindResourceEx(hMod, szResourceType, szResourceName, wLanguage);
   if (hRsrc) {
      HGLOBAL hGlobal = ::LoadResource(hMod, hRsrc);
      if (hGlobal) {
         dwSize = ::SizeofResource(hMod, hRsrc);
         if (dwSize) {
            pbRes = (const BYTE* )::LockResource(hGlobal);
            if (!pbRes)
               dwSize = 0;
         }
      }
   }
   return pbRes;
}

BOOL ExtractResource_internal(
   IN const BYTE *pbResource,
   IN DWORD       nbSize,
   IN HMODULE     hMod,
   IN LPCTSTR     szResourceType,
   IN LPCTSTR     szResourceName,
   IN WORD        wLanguage,
   IN LPCTSTR     szFileName
) {
   DWORD dwErrCode = NO_ERROR;
   BOOL bRes = ((pbResource != NULL) && (nbSize!=0));
   if (!bRes) {
      dwErrCode = ERROR_INVALID_PARAMETER;
   } else {
      switch ((ULONG_PTR)szResourceType) {



      case RT_GROUP_ICON:
      #ifdef __FILE__ICONS__
         //bRes = ::ExtractIco(hMod, (WORD)szResourceName, szFileName);
         bRes = ::ExtractIco(hMod, szResourceName, wLanguage, szFileName);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         }
      #else // __FILE__ICONS__
         bRes = FALSE;
         #pragma message("  >>>Please include 'Icon.h' to the project - add header to this file (" __FILE__ ") or 'StdAfx.h' file...")
         dwErrCode = ERROR_CALL_NOT_IMPLEMENTED;
      #endif // __FILE__ICONS__
         break;



      case RT_ICON:
         bRes = IS_INTRESOURCE(szResourceName);
         if (!bRes) {
            // resouce name   WORD'
            dwErrCode = ERROR_INVALID_PARAMETER;
         } else {
      #ifdef __FILE__ICONS__
            bRes = ::ExtractIco(hMod, (WORD)szResourceName, szFileName);
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
      #else // __FILE__ICONS__
            bRes = FALSE;
            #pragma message("  >>>Please include 'Icon.h' to the project - add header to this file (" __FILE__ ") or 'StdAfx.h' file...")
            dwErrCode = ERROR_CALL_NOT_IMPLEMENTED;
      #endif // __FILE__ICONS__
         }
         break;



      case RT_BITMAP:
      #ifdef __FILE__BITMAPS__
         if (wLanguage)
            bRes = ::ExtractBmp(hMod, szResourceName, wLanguage, szFileName);
         else
            bRes = ::ExtractBmp(hMod, szResourceName, szFileName);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         }
      #else // __FILE__BITMAPS__
         bRes = FALSE;
         #pragma message("  >>>Please include 'Bitmap.h' to the project - add header to this file (" __FILE__ ") or 'StdAfx.h' file...")
         dwErrCode = ERROR_CALL_NOT_IMPLEMENTED;
      #endif // __FILE__BITMAPS__
         break;



      default:
         {
            HANDLE hFile = ::CreateFile(
               szFileName,
               GENERIC_WRITE,
               FILE_SHARE_READ,
               NULL,
               CREATE_ALWAYS,
               FILE_ATTRIBUTE_NORMAL,
               NULL
            );
            bRes = (hFile != INVALID_HANDLE_VALUE);
            if (!bRes) {
               dwErrCode = ::GetLastError();
            } else {
               DWORD dwNOBW = 0;
               bRes = (::WriteFile(hFile, pbResource, nbSize, &dwNOBW, NULL) && (nbSize == dwNOBW));
               if (!bRes) {
                  dwErrCode = ::GetLastError();
               }
               ::CloseHandle(hFile); hFile = NULL;
            }
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}

BOOL ExtractResource(
   IN HMODULE hMod,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   IN LPCTSTR szFileName
) {
   ::SetLastError(NO_ERROR);
   DWORD dwSize = 0;
   const BYTE *pbResource = GetResource(hMod, szResourceType, szResourceName, dwSize);
   BOOL bRes = (pbResource && dwSize);
   if (bRes) {
      bRes = ExtractResource_internal(pbResource, dwSize, hMod, szResourceType, szResourceName, 0, szFileName);
   }
   return bRes;
}

BOOL ExtractResource(
   IN HMODULE hMod,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   IN WORD    wLanguage,
   IN LPCTSTR szFileName
) {
   ::SetLastError(NO_ERROR);
   DWORD dwSize = 0;
   const BYTE *pbResource = GetResource(hMod, szResourceType, szResourceName, wLanguage, dwSize);
   BOOL bRes = (pbResource && dwSize);
   if (bRes) {
      bRes = ExtractResource_internal(pbResource, dwSize, hMod, szResourceType, szResourceName, wLanguage, szFileName);
   }
   return bRes;
}



//////////////////////////////////////////////////////////////////////////////

namespace nsInternalResource {
   //////////////////////////////////////////////////////////////
   //                  ...
   //////////////////////////////////////////////////////////////

   template <class TResData>
   struct CResourceDataArr {
      TResData *m_pArr;
      WORD const m_wLenArr;
      WORD m_i;
      CResourceDataArr(TResData *pArr, const WORD wLenBuf) : m_pArr(pArr), m_wLenArr(wLenBuf), m_i(0)  {}
   };
   BOOL CALLBACK EnumResLangProc_Arr(HMODULE hMod, LPCTSTR lpszType, LPCTSTR lpszName, WORD wIDLanguage, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      CResourceDataArr<CRcEx> &data = *(CResourceDataArr<CRcEx>*)lParam;
      if (data.m_i < data.m_wLenArr) {
         data.m_pArr[data.m_i].setType(lpszType);
         data.m_pArr[data.m_i].setName(lpszName);
         data.m_pArr[data.m_i].setLang(wIDLanguage);
         data.m_i++;
      } else {
         ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return FALSE;
      }
      return TRUE;
   }
   BOOL CALLBACK EnumResLangProc_Arr_NoType(HMODULE hMod, LPCTSTR /*lpszType*/, LPCTSTR lpszName, WORD wIDLanguage, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      CResourceDataArr<CRcNameEx> &data = *(CResourceDataArr<CRcNameEx>*)lParam;
      if (data.m_i < data.m_wLenArr) {
         data.m_pArr[data.m_i].setName(lpszName);
         data.m_pArr[data.m_i].setLang(wIDLanguage);
         data.m_i++;
      } else {
         ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return FALSE;
      }
      return TRUE;
   }
   BOOL CALLBACK EnumResLangProc_Arr_NoTypeNoName(HMODULE hMod, LPCTSTR /*lpszType*/, LPCTSTR /*lpszName*/, WORD wIDLanguage, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      CResourceDataArr<CRcLang> &data = *(CResourceDataArr<CRcLang>*)lParam;
      if (data.m_i < data.m_wLenArr) {
         data.m_pArr[data.m_i].setLang(wIDLanguage);
         data.m_i++;
      } else {
         ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return FALSE;
      }
      return TRUE;
   }
   //////////////////////////////////////////////////////////////
   BOOL CALLBACK EnumResNameProc_NameArr(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      CResourceDataArr<CRc> &data = *(CResourceDataArr<CRc>*)lParam;
      if (data.m_i < data.m_wLenArr) {
         data.m_pArr[data.m_i].setType(lpszType);
         data.m_pArr[data.m_i].setName(lpszName);
         data.m_i++;
      } else {
         ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return FALSE;
      }
      return TRUE;
   }
   BOOL CALLBACK EnumResNameProc_NameArr_NoType(HMODULE hMod, LPCTSTR /*lpszType*/, LPTSTR lpszName, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      CResourceDataArr<CRcName> &data = *(CResourceDataArr<CRcName>*)lParam;
      if (data.m_i < data.m_wLenArr) {
         data.m_pArr[data.m_i].setName(lpszName);
         data.m_i++;
      } else {
         ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return FALSE;
      }
      return TRUE;
   }
   BOOL CALLBACK EnumResNameProc_NameArrLang(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG_PTR lParam) {
      return ::EnumResourceLanguages(hMod, lpszType, lpszName, EnumResLangProc_Arr, lParam);
   }
   BOOL CALLBACK EnumResNameProc_NameArrLang_NoType(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG_PTR lParam) {
      return ::EnumResourceLanguages(hMod, lpszType, lpszName, EnumResLangProc_Arr_NoType, lParam);
   }
   //////////////////////////////////////////////////////////////
   BOOL CALLBACK EnumResTypeProc_TypeArr(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      CResourceDataArr<CRcType> &data = *(CResourceDataArr<CRcType>*)lParam;
      if (data.m_i < data.m_wLenArr) {
         data.m_pArr[data.m_i].setType(lpszType);
         data.m_i++;
      } else {
         ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
         return FALSE;
      }
      return TRUE;
   }
   BOOL CALLBACK EnumResTypeProc_TypeArrName(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
      return ::EnumResourceNames(hMod, lpszType, EnumResNameProc_NameArr, lParam);
   }
   BOOL CALLBACK EnumResTypeProc_TypeArrNameLang(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
      return ::EnumResourceNames(hMod, lpszType, EnumResNameProc_NameArrLang, lParam);
   }
   //////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////
   BOOL CALLBACK EnumResLangProc_Count(HMODULE hMod, LPCTSTR lpszType, LPCTSTR lpszName, WORD wIDLanguage, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      WORD &wRes = *(WORD*)lParam;
      wRes++;
      return TRUE;
   }
   //////////////////////////////////////////////////////////////
   BOOL CALLBACK EnumResNameProc_NameCount(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      WORD &wRes = *(WORD*)lParam;
      wRes++;
      return TRUE;
   }
   BOOL CALLBACK EnumResNameProc_NameCountLang(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG_PTR lParam) {
      return ::EnumResourceLanguages(hMod, lpszType, lpszName, EnumResLangProc_Count, lParam);
   }
   //////////////////////////////////////////////////////////////
   BOOL CALLBACK EnumResTypeProc_TypeCount(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
      if (!lParam) return FALSE;
      WORD &wRes = *(WORD*)lParam;
      wRes++;
      return TRUE;
   }
   BOOL CALLBACK EnumResTypeProc_TypeCountName(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
      return ::EnumResourceNames(hMod, lpszType, EnumResNameProc_NameCount, lParam);
   }
   BOOL CALLBACK EnumResTypeProc_TypeCountNameLang(HMODULE hMod, LPTSTR lpszType, LONG_PTR lParam) {
      return ::EnumResourceNames(hMod, lpszType, EnumResNameProc_NameCountLang, lParam);
   }
   //////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////

   BOOL CALLBACK EnumResNameProc_FindNewResouceName(HMODULE hMod, LPCTSTR lpszType, LPTSTR lpszName, LONG_PTR lParam) {
      if (IS_INTRESOURCE(lpszName)) {
         WORD &wNew = *(WORD*)lParam;
         wNew = max((WORD)lpszName, wNew);
      }
      return TRUE;
   }
} // nsInternalResource

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

WORD GetResourceCount(HMODULE hMod, BOOL bLang) { //        (bLang -   )
   WORD wRes = 0;
   BOOL bRes = ::EnumResourceTypes(hMod, bLang ? nsInternalResource::EnumResTypeProc_TypeCountNameLang :
                                                 nsInternalResource::EnumResTypeProc_TypeCountName, (LONG_PTR)&wRes);
   return bRes ? wRes : 0;
}
WORD GetResourceTypeCount(HMODULE hMod) { //  
   WORD wRes = 0;
   BOOL bRes = ::EnumResourceTypes(hMod, nsInternalResource::EnumResTypeProc_TypeCount, (LONG_PTR)&wRes);
   return bRes ? wRes : 0;
}
WORD GetResourceNameCount(HMODULE hMod, LPCTSTR szResourceType, BOOL bLang) {  //      (bLang -    )
   WORD wRes = 0;
   BOOL bRes = ::EnumResourceNames(hMod, szResourceType, bLang ? nsInternalResource::EnumResNameProc_NameCountLang :
                                                                 nsInternalResource::EnumResNameProc_NameCount, (LONG_PTR)&wRes);
   return bRes ? wRes : 0;
}
WORD GetResourceLangCount(HMODULE hMod, LPCTSTR szResourceType, LPCTSTR szResourceName) { //     (     )
   WORD wRes = 0;
   BOOL bRes = ::EnumResourceLanguages(hMod, szResourceType, szResourceName, nsInternalResource::EnumResLangProc_Count, (LONG_PTR)&wRes);
   return bRes ? wRes : 0;
}

BOOL GetResources(IN HMODULE hMod, OUT CRc *pArr, IN OUT WORD &wLenBuf) { //       
   WORD wNeed = GetResourceCount(hMod, FALSE);
   if (wLenBuf < wNeed) {
      wLenBuf = wNeed;
      ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
      return FALSE;
   }
   if (pArr == NULL) {
      ::SetLastError(ERROR_INVALID_PARAMETER);
      return FALSE;
   }
   nsInternalResource::CResourceDataArr<CRc> arr(pArr, wLenBuf);
   BOOL bRes = ::EnumResourceTypes(hMod, nsInternalResource::EnumResTypeProc_TypeArrName, (LONG_PTR)&arr);
   wLenBuf = arr.m_i; //      
   return bRes;
}

BOOL GetResourcesEx(IN HMODULE hMod, OUT CRcEx *pArr, IN WORD &wLenBuf) { //          
   WORD wNeed = GetResourceCount(hMod, TRUE);
   if (wLenBuf < wNeed) {
      wLenBuf = wNeed;
      ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
      return FALSE;
   }
   if (pArr == NULL) {
      ::SetLastError(ERROR_INVALID_PARAMETER);
      return FALSE;
   }
   nsInternalResource::CResourceDataArr<CRcEx> arr(pArr, wLenBuf);
   BOOL bRes = ::EnumResourceTypes(hMod, nsInternalResource::EnumResTypeProc_TypeArrNameLang, (LONG_PTR)&arr);
   wLenBuf = arr.m_i; //      
   return bRes;
}

BOOL GetResourceTypes(IN HMODULE hMod, OUT CRcType *pArr, IN OUT WORD &wLenBuf) { //    
   WORD wNeed = GetResourceTypeCount(hMod);
   if (wLenBuf < wNeed) {
      wLenBuf = wNeed;
      ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
      return FALSE;
   }
   if (pArr == NULL) {
      ::SetLastError(ERROR_INVALID_PARAMETER);
      return FALSE;
   }
   nsInternalResource::CResourceDataArr<CRcType> arr(pArr, wLenBuf);
   BOOL bRes = ::EnumResourceTypes(hMod, nsInternalResource::EnumResTypeProc_TypeArr, (LONG_PTR)&arr);
   wLenBuf = arr.m_i; //      
   return bRes;
}

BOOL GetResourceNames(IN HMODULE hMod, IN LPCTSTR szResourceType, OUT CRcName *pArr, IN OUT WORD &wLenBuf) { //       
   WORD wNeed = GetResourceNameCount(hMod, szResourceType, FALSE);
   if (wLenBuf < wNeed) {
      wLenBuf = wNeed;
      ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
      return FALSE;
   }
   if (pArr == NULL) {
      ::SetLastError(ERROR_INVALID_PARAMETER);
      return FALSE;
   }
   nsInternalResource::CResourceDataArr<CRcName> arr(pArr, wLenBuf);
   BOOL bRes = ::EnumResourceNames(hMod, szResourceType, nsInternalResource::EnumResNameProc_NameArr_NoType, (LONG_PTR)&arr);
   wLenBuf = arr.m_i; //      
   return bRes;
}

BOOL GetResourceNamesEx(IN HMODULE hMod, IN LPCTSTR szResourceType, OUT CRcNameEx *pArr, IN OUT WORD &wLenBuf) { //          
   WORD wNeed = GetResourceNameCount(hMod, szResourceType, TRUE);
   if (wLenBuf < wNeed) {
      wLenBuf = wNeed;
      ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
      return FALSE;
   }
   if (pArr == NULL) {
      ::SetLastError(ERROR_INVALID_PARAMETER);
      return FALSE;
   }
   nsInternalResource::CResourceDataArr<CRcNameEx> arr(pArr, wLenBuf);
   BOOL bRes = ::EnumResourceNames(hMod, szResourceType, nsInternalResource::EnumResNameProc_NameArrLang_NoType, (LONG_PTR)&arr);
   wLenBuf = arr.m_i; //      
   return bRes;
}

BOOL GetResourceLanguages(IN HMODULE hMod, IN LPCTSTR szResourceType, IN LPCTSTR szResourceName, OUT CRcLang *pArr, IN OUT WORD &wLenBuf) { //      
   WORD wNeed = GetResourceLangCount(hMod, szResourceType, szResourceName);
   if (wLenBuf < wNeed) {
      wLenBuf = wNeed;
      ::SetLastError(ERROR_INSUFFICIENT_BUFFER);
      return FALSE;
   }
   if (pArr == NULL) {
      ::SetLastError(ERROR_INVALID_PARAMETER);
      return FALSE;
   }
   nsInternalResource::CResourceDataArr<CRcLang> arr(pArr, wLenBuf);
   BOOL bRes = ::EnumResourceLanguages(hMod, szResourceType, szResourceName, nsInternalResource::EnumResLangProc_Arr_NoTypeNoName, (LONG_PTR)&arr);
   wLenBuf = arr.m_i; //      
   return bRes;
}

BOOL AddResource(IN LPCTSTR szModule, IN LPCTSTR szResType, IN LPCTSTR szResName, IN WORD wLang, IN const BYTE* pbData, IN DWORD cbSizeData) {
   //   ,  szResType  szResName    WORD,
   // ..       MAKEINTRESOURCE.
   // ,   -   ,       
   // FindResource(Ex)   ( ERROR_RESOURCE_TYPE_NOT_FOUND).
   //  ,   szResName   ,     (???)
   //     szResType.
   DWORD dwAttr = ::GetFileAttributes(szModule);
   BOOL bRes = ((0xFFFFFFFF != dwAttr) && !(dwAttr & FILE_ATTRIBUTE_DIRECTORY));
   DWORD dwErrCode = NO_ERROR;
   if (!bRes) {
      if (0xFFFFFFFF==dwAttr)
         dwErrCode = ::GetLastError();
      else
         dwErrCode = ERROR_FILE_NOT_FOUND;
   } else {
      HANDLE hUpdate = ::BeginUpdateResource(szModule, FALSE);
      bRes = !!hUpdate;
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         bRes = ::UpdateResource(hUpdate, szResType, szResName, wLang, (LPBYTE)pbData, cbSizeData);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         } else {
            bRes = ::EndUpdateResource(hUpdate, FALSE);
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}
BOOL AddResource(IN LPCTSTR szModule, IN WORD wResType, IN WORD wResName, IN WORD wLang, IN const BYTE* pbData, IN DWORD cbSizeData) {
   return AddResource(szModule, MAKEINTRESOURCE(wResType), MAKEINTRESOURCE(wResName), wLang, pbData, cbSizeData);
}

BOOL DelResource(IN LPCTSTR szModule, IN LPCTSTR szResType, IN LPCTSTR szResName, IN WORD wLang) {
   DWORD dwAttr = ::GetFileAttributes(szModule);
   BOOL bRes = ((0xFFFFFFFF != dwAttr) && !(dwAttr & FILE_ATTRIBUTE_DIRECTORY));
   DWORD dwErrCode = NO_ERROR;
   if (!bRes) {
      if (0xFFFFFFFF==dwAttr)
         dwErrCode = ::GetLastError();
      else
         dwErrCode = ERROR_FILE_NOT_FOUND;
   } else {
      HANDLE hUpdate = ::BeginUpdateResource(szModule, FALSE);
      bRes = !!hUpdate;
      if (!bRes) {
         dwErrCode = ::GetLastError();
      } else {
         bRes = ::UpdateResource(hUpdate, szResType, szResName, wLang, NULL, 0);
         if (!bRes) {
            dwErrCode = ::GetLastError();
         } else {
            bRes = ::EndUpdateResource(hUpdate, FALSE);
            if (!bRes) {
               dwErrCode = ::GetLastError();
            }
         }
      }
   }
   ::SetLastError(dwErrCode);
   return bRes;
}
BOOL DelResource(IN LPCTSTR szModule, IN WORD wResType, IN WORD wResName, IN WORD wLang) {
   return DelResource(szModule, MAKEINTRESOURCE(wResType), MAKEINTRESOURCE(wResName), wLang);
}

WORD FindNewResouceName(IN HMODULE hMod, IN LPCTSTR szType) {
   DWORD dwErrCode = NO_ERROR;
   WORD wRes = 0;
   WORD wNew = 1;
   BOOL bRes = ::EnumResourceNames(hMod, szType, nsInternalResource::EnumResNameProc_FindNewResouceName, (LONG_PTR)&wNew);
   if (!bRes) {
      dwErrCode = ::GetLastError();
      if (ERROR_RESOURCE_TYPE_NOT_FOUND == dwErrCode) {
         dwErrCode = NO_ERROR;
         wRes = 1;
      }
   } else {
      wRes = wNew+1;
   }
   ::SetLastError(dwErrCode);
   return wRes;
}
WORD FindNewResouceName(IN LPCTSTR szModule, IN LPCTSTR szType) {
   HMODULE hMod = ::LoadLibraryEx(szModule, NULL, LOAD_LIBRARY_AS_DATAFILE);
   if (!hMod) {
      return 0;
   } else {
      WORD wRes = FindNewResouceName(hMod, szType);
      DWORD dwErrCode = NO_ERROR;
      if (0==wRes) {
         dwErrCode = ::GetLastError();
      }
      ::FreeLibrary(hMod);
      ::SetLastError(dwErrCode);
      return wRes;
   }
}