////////////////////////////////////////////////////////////////////////////////
// File name: Resources.h
// Author: Sergey Krivulya (Ce p) - KSerg (SeregaLBN)
// e-mail: Sergey_Krivulya@UkrPost.Net
// ICQ: 72099167
// Date: 2011 06 18
//
// Description:   
// Copyright (C) 2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__RESOURCES__
#define __FILE__RESOURCES__

#if _MSC_VER > 1000
   #pragma once
#endif // _MSC_VER > 1000

#ifndef USE_CSTRING_CLASS
   #if defined(__AFX_H__) || defined(_WTL_USE_CSTRING) || defined(__ATLSTR_H__) || defined(__FILE__CSTRINGKS__)
      #define USE_CSTRING_CLASS
   #endif
#endif

const BYTE* GetResource( //     
   IN HMODULE hModule,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   OUT DWORD &dwSize
);
const BYTE* GetResource( //     
   IN HMODULE hModule,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   IN WORD wLanguage,
   OUT DWORD &dwSize
);
BOOL ExtractResource( //      
   IN HMODULE hModule,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   IN LPCTSTR szFileName
);
BOOL ExtractResource( //         
   IN HMODULE hModule,
   IN LPCTSTR szResourceType,
   IN LPCTSTR szResourceName,
   IN WORD    wLanguage,
   IN LPCTSTR szFileName
);

WORD GetResourceCount    (HMODULE hMod, BOOL bLang); //        (bLang -   )
WORD GetResourceTypeCount(HMODULE hMod); //  
WORD GetResourceNameCount(HMODULE hMod, LPCTSTR szResourceType, BOOL bLang); //      (bLang -    )
WORD GetResourceLangCount(HMODULE hMod, LPCTSTR szResourceType, LPCTSTR szResourceName); //     (     )

class CRcType {
protected:
   LPTSTR m_szType;
public:
   CRcType() : m_szType(NULL) {}
   CRcType(LPCTSTR szType) : m_szType(NULL) { setType(szType); }
   CRcType(const CRcType& copy) : m_szType(NULL) { setType(copy.getType_raw()); }
   virtual ~CRcType() { if (m_szType && !IS_INTRESOURCE(m_szType)) { ::free(m_szType); m_szType = NULL; } }
   void setType(LPCTSTR szType) {
      if (IS_INTRESOURCE(szType)) m_szType = MAKEINTRESOURCE(szType);
      else m_szType = ::_tcsdup(szType);
   }
   LPCTSTR getType_raw() const { return m_szType; }
   BOOL isValid() const { return (m_szType != NULL); }
#ifdef USE_CSTRING_CLASS
   CString getTypeValue() { return IS_INTRESOURCE(m_szType) ? ::Format(_T("%i"), (int)(WORD)m_szType) : m_szType; }
   CString toString() { return getTypeValue(); }
#endif
};

class CRcName {
protected:
   LPTSTR m_szName;
public:
   CRcName() : m_szName(NULL) {}
   CRcName(LPCTSTR szName) : m_szName(NULL) { setName(szName); }
   CRcName(const CRcName& copy) : m_szName(NULL) { setName(copy.getName_raw()); }
   __forceinline virtual ~CRcName() { if (m_szName && !IS_INTRESOURCE(m_szName)) { ::free(m_szName); m_szName = NULL; } }
   void setName(LPCTSTR szName) {
      if (IS_INTRESOURCE(szName)) m_szName = MAKEINTRESOURCE(szName);
      else m_szName = ::_tcsdup(szName);
   }
   LPCTSTR getName_raw() const { return m_szName; }
   BOOL isValid() const { return (m_szName != NULL); }
#ifdef USE_CSTRING_CLASS
   CString getNameValue() { return IS_INTRESOURCE(m_szName) ? ::Format(_T("%i"), (int)(WORD)m_szName) : m_szName; }
   CString toString() { return getNameValue(); }
#endif
};

class CRcLang {
protected:
   WORD m_wIDLanguage;
public:
   CRcLang() : m_wIDLanguage(0) {}
   CRcLang(WORD wIDLanguage) : m_wIDLanguage(wIDLanguage) {}
   CRcLang(const CRcLang& copy) : m_wIDLanguage(0) { setLang(copy.getLang()); }
   WORD getLang() const { return m_wIDLanguage; }
   void setLang(WORD wIDLanguage) { m_wIDLanguage = wIDLanguage; }
#ifdef USE_CSTRING_CLASS
   CString getLocaleValue() { return Locale(m_wIDLanguage, LOCALE_SABBREVLANGNAME); }
   CString toString() { return ::Format(_T("%i"), (int)m_wIDLanguage); }
#endif
};

class CRcNameEx: public CRcName, public CRcLang {
public:
   CRcNameEx() {}
   CRcNameEx(const CRcNameEx& copy) {
      setName(copy.getName_raw());
      setLang(copy.getLang());
   }
   CRcNameEx(LPCTSTR szName, WORD wIDLanguage) {
      setName(szName);
      setLang(wIDLanguage);
   }
#ifdef USE_CSTRING_CLASS
   CString toString() {
      CRcName &name = *this;
      CRcLang &lang = *this;
      return ::Format(_T("name[%s]; lang[%s]"), (LPCTSTR)name.toString(), (LPCTSTR)lang.toString());
   }
#endif
};

class CRc: public CRcType, public CRcName {
public:
   CRc() {}
   CRc(const CRc& copy) {
      setType(copy.getType_raw());
      setName(copy.getName_raw());
   }
   CRc(LPCTSTR szType, LPCTSTR szName) {
      setType(szType);
      setName(szName);
   }
   bool isValid() {
      CRcType &type = *this;
      CRcName &name = *this;
      return type.isValid() && name.isValid();
   }
#ifdef USE_CSTRING_CLASS
   CString toString() {
      CRcType &type = *this;
      CRcName &name = *this;
      return ::Format(_T("type[%s]; name[%s]"), (LPCTSTR)type.toString(), (LPCTSTR)name.toString());
   }
#endif
};

class CRcEx: public CRcType, public CRcName, public CRcLang {
public:
   CRcEx() {}
   CRcEx(const CRcEx& copy) {
      setType(copy.getType_raw());
      setName(copy.getName_raw());
      setLang(copy.getLang());
   }
   CRcEx(LPCTSTR szType, LPCTSTR szName, WORD wIDLanguage) {
      setType(szType);
      setName(szName);
      setLang(wIDLanguage);
   }
   bool isValid() {
      CRcType &type = *this;
      CRcName &name = *this;
      return type.isValid() && name.isValid();
   }
#ifdef USE_CSTRING_CLASS
   CString toString() {
      CRcType &type = *this;
      CRcName &name = *this;
      CRcLang &lang = *this;
      return ::Format(_T("type[%s]; name[%s]; lang=[%s]"), (LPCTSTR)type.toString(), (LPCTSTR)name.toString(), (LPCTSTR)lang.toString());
   }
#endif
};

BOOL GetResources        (IN HMODULE hMod,                                                       OUT CRc       *pResources, IN OUT WORD &wLenBuf); //       
BOOL GetResourcesEx      (IN HMODULE hMod,                                                       OUT CRcEx     *pResources, IN OUT WORD &wLenBuf); //          
BOOL GetResourceTypes    (IN HMODULE hMod,                                                       OUT CRcType   *pTypes    , IN OUT WORD &wLenBuf); //    
BOOL GetResourceNames    (IN HMODULE hMod, IN LPCTSTR szResourceType,                            OUT CRcName   *pNames    , IN OUT WORD &wLenBuf); //       
BOOL GetResourceNamesEx  (IN HMODULE hMod, IN LPCTSTR szResourceType,                            OUT CRcNameEx *pNames    , IN OUT WORD &wLenBuf); //          
BOOL GetResourceLanguages(IN HMODULE hMod, IN LPCTSTR szResourceType, IN LPCTSTR szResourceName, OUT CRcLang   *pLanguages, IN OUT WORD &wLenBuf); //      

//  
BOOL AddResource(IN LPCTSTR szModule, IN WORD    wResType , IN WORD    wResName, IN WORD wLang, IN const BYTE* pbData, IN DWORD cbSizeData);
BOOL DelResource(IN LPCTSTR szModule, IN LPCTSTR szResType, IN LPCTSTR szResName, IN WORD wLang);
BOOL DelResource(IN LPCTSTR szModule, IN WORD    wResType , IN WORD    wResName, IN WORD wLang);
WORD FindNewResouceName(IN HMODULE hMod    , IN LPCTSTR szType); //  ,   ,    
WORD FindNewResouceName(IN LPCTSTR szModule, IN LPCTSTR szType); //     (. GetLastError)

#endif // __FILE__RESOURCES__
