// MainDlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

#include "./Lib/Draw/GradientColors.h"

class CMainDlg : public CDialogImpl<CMainDlg>
{
private:
   const SIZE m_sizeView;

   HBITMAP m_hBmpView;
   HDC m_hDC;
   HGDIOBJ m_hBmpViewOld;

   GradientColors m_Grad; // gradient colors fill

   UINT m_uMsgAsyncActivate, m_uMsgAsyncSelectFolder;

   CString m_strModule; //   ,    
   HMODULE m_hMod;      //   
   BOOL    m_bLoadLib;  //    ?

   CString m_strImageFolder;

   struct CFrameInfo {
      HICON m_hIco;
      SIZE  m_sizeIco;
      CFrameInfo() : m_hIco(NULL) { m_sizeIco.cx=m_sizeIco.cy=0; }
      CFrameInfo(HICON hIco, SIZE sizeIco) : m_hIco(hIco), m_sizeIco(sizeIco) {}
   };
   typedef std::vector<CFrameInfo> ARR_IcoFrames;
   struct CIcoInfo {
      ARR_IcoFrames m_frames;
      CString       m_name; // file or resource name
      CIcoInfo() {}
      CIcoInfo(ARR_IcoFrames frames, CString name) : m_frames(frames), m_name(name) {}
   };
   typedef std::vector<CIcoInfo> ARR_Icons;

   ARR_Icons m_arrIcoRcAllFrames;             //    
   ARR_Icons m_arrIcoRcAllFramesFromGroup;    //      (  )
   ARR_Icons m_arrIcoRcAllFramesFromGroupLng; //      (  )
   ARR_Icons m_arrIcoAllDirFrames;            //     

   struct CBmpInfo {
      HBITMAP m_hBmp;
      SIZE    m_sizeBmp;
      CString m_name; // file or resource name
      CBmpInfo() : m_hBmp(NULL) { m_sizeBmp.cx=m_sizeBmp.cy=0; }
      CBmpInfo(HBITMAP hBmp, SIZE sizeBmp, CString name) : m_hBmp(hBmp), m_sizeBmp(sizeBmp), m_name(name) {
         _ASSERT(hBmp);
      }
   };
   typedef std::vector<CBmpInfo> ARR_Bitmaps;

   ARR_Bitmaps m_arrBmpRcAll;    //     (  )
   ARR_Bitmaps m_arrBmpRcAllLng; //     (  )
   ARR_Bitmaps m_arrBmpDirAll;   //    


public:
   CMainDlg();
   enum { IDD = IDD_MAINDLG };

   BEGIN_MSG_MAP_EX(CMainDlg)
      MSG_WM_INITDIALOG(OnInitDialog)
      MSG_WM_ACTIVATEAPP(OnActivateApp)
      MSG_WM_ACTIVATE(OnActivate)
      MSG_WM_SHOWWINDOW(OnShowWindow)
      MSG_WM_DESTROY(OnDestroy)
      MSG_WM_HSCROLL(OnHScroll)
      MSG_WM_VSCROLL(OnVScroll)
      MSG_WM_PAINT(OnPaint)
      MSG_WM_ERASEBKGND(OnEraseBkgnd)
      MSG_WM_TIMER(OnTimer)
      MSG_WM_MOUSEWHEEL(OnMouseWheel)
      MSG_WM_CHANGEUISTATE(OnChangeUIState)
      MESSAGE_HANDLER_EX(m_uMsgAsyncActivate, OnAsyncActive)
      MESSAGE_HANDLER_EX(m_uMsgAsyncSelectFolder, OnAsyncSelectFolder)

      COMMAND_ID_HANDLER_EX(IDOK, OnClose)
      COMMAND_ID_HANDLER_EX(IDCANCEL, OnClose)
   END_MSG_MAP()

   BOOL OnInitDialog(HWND, LPARAM);
   void OnDestroy();
   void OnActivateApp(BOOL bActive, DWORD dwThreadID);
   void OnActivate(UINT nState, BOOL bMinimized, CWindow wndOther);
   LRESULT OnAsyncActive(UINT uMsg, WPARAM wParam, LPARAM lParam);
   LRESULT OnAsyncSelectFolder(UINT uMsg, WPARAM wParam, LPARAM lParam);
   void OnShowWindow(BOOL bShow, UINT nStatus);
   void OnChangeUIState(WORD nAction, WORD nState);
   void OnClose(UINT, int, CWindow);
   void OnPaint(CDCHandle dc);
   BOOL OnEraseBkgnd(CDCHandle dc) { return TRUE; }
   void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar pScrollBar);
   void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar pScrollBar);
   BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
   void OnTimer(UINT_PTR nIDEvent);

   SIZE getScrollOffset();

   void SelectFolder();
   void SelectAndLoadMod();
   void FreeMod();

   void LoadIcoFromModule();
   void LoadIcoFromDir();
   void LoadBmpFromModule();
   void LoadBmpFromDir();

   void CreateCompatibleDC_and_Bmp();
   void PaintBackground();
   void PaintImages();

};

//#ifdef _DEBUG
//#include "./Diagnostics/Helper.h"
//
//class CMainDlgDebug : public CMainDlg {
//public:
//   virtual BOOL ProcessWindowMessage(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT& lResult, DWORD dwMsgMapID) {
//      ::OutputDebugStr(::Format(_T("CMainDlg: uMsg=[%s]\r\n"), (LPCTSTR)nsHelper::WindowMessage(uMsg)));
//      return __super::ProcessWindowMessage(hWnd, uMsg, wParam, lParam, lResult, dwMsgMapID);
//   }
//};
//#endif